import urllib2
from urllib import urlencode
from warnings import warn
import settings

baseurl = settings.places_baseurl 

class Unlock:
    def __init__(self,key=None,gazetteer=None):
        self.apikey = key
        self.gazetteer = gazetteer

class Places(Unlock):
    
    def ask_service(self,request,params):
        if self.apikey is not None:
            params['key'] = self.apikey
        url = baseurl+request+'?'+urlencode(params)
        try:
            r = urllib2.urlopen(url)
            return r.read()
        except urllib2.HTTPError, e:
            warn(e)
            return None

    def nameSearch(self,name=None,format=None):
        """Look up a name, get back short description in format
        one of ['json','kml',?]"""

        params = {'name':name,'format':format, 'gazetteer':self.pick_gazetteer()}   
        results = self.ask_service('nameSearch',params)
        return results

    def spatialNameSearch(self,name=None,format=None,minx=None,miny=None,maxx=None,maxy=None,operator=None):
        """Look up a name, get back short description in format
        one of ['json','kml',?]"""

        if operator is None: operator = 'within'

        params = {'name':name,'format':format, 'gazetteer':self.pick_gazetteer(),'minx':minx,'maxx':maxx,'miny':miny,'maxy':maxy,'operator':operator}

        return self.ask_service('spatialNameSearch',params)        


    def nameAndFeatureSearch(self,name=None,featureType=None,format=None):
        """Look up a name but limit results by specified feature type(s)?"""

        params = {'name':name,'format':format,'featureType':featureType,'gazetteer':self.pick_gazetteer()}
        return self.ask_service('nameAndFeatureSearch',params)

    def featureTypes(self,superclass=None):
        """Is this written yet."""
        return self.ask_service('featureTypes')
        
    def footprintLookup(self,id=None,format=None):
        """Given an identifier get back a shape (if one is available)"""
        params = {'gazetteer':self.pick_gazetteer(),'identifier':id,'format':format}

        return self.ask_service('footprintLookup',params)
         
        #return r.read()
                                                                 

    def pick_gazetteer(self,gazetteer=None):
        """Choose geonames by default, if a gazetteer was not specified.
        Optionally pass in a gazetteer to use here.
        """
        if gazetteer is not None:
            self.gazetteer = gazetteer
        if self.gazetteer is None:
            if self.apikey is not None:
                self.gazetteer = 'os'    
            else:
                self.gazetteer = 'geonames'
        return self.gazetteer

    


if __name__ == '__main__':

    p = Places(key=None)
    output = p.nameSearch(name='Dunfermline',format='json')
    print output         
    



from urllib import urlencode
from urllib2 import urlopen, Request
import settings

baseurl = settings.text_baseurl

class Text:
    def places(self,doc,params=None):
        values = {}
        if params is not None:
            values = params
        if 'gazetteer' not in values:
            values['gazetteer'] = 'geonames'
        if 'type' not in values:
            values['type'] = 'plain'
        values['document'] = doc
        data = urlencode(values)
        req = Request(baseurl,data)
        res = urlopen(req)
        return res.read()





import setuptools
from setuptools import setup, find_packages

setup(name="Unlock",
      version="0.1",
      description="Client for the Unlock web services",
      long_description="Client code to make calls to the RESTful API at http://unlock.edina.ac.uk for gazetteer, place search and geographic reference text extraction",
      packages=find_packages(),
      #package_data={'': '*.html'},
      install_requires=[],
      author='Jo Walsh',
      author_email='jo.walsh@ed.ac.uk',
      url='http://unlock.edina.ac.uk/code.html',
)	



from unlock import Places, Text
import sys
sys.path.append('/home/jwalsh/unlock/py/src')
import unittest
import simplejson

class TestSequenceFunctions(unittest.TestCase):

    def setUp(self):
        self.name = 'Dunfermline'
        self.formats = ['json','kml','georss']  
    
    def testNameSearch(self):
        # simple name search
        p = Places() 
        xml = p.nameSearch(self.name)

    def testSpatialNameSearch(self):
        # bounding box name search
        bbox = {'minx' : -8, 'maxx': 1.5, 'miny' :54, 'maxy':62}
        places = Places()
        for f in self.formats:
            res = places.spatialNameSearch(minx=bbox['minx'],miny=bbox['miny'],maxx=bbox['maxx'],maxy=bbox['maxy'],name=self.name,format=f)
            if f == 'json':
                json = simplejson.loads(res)
                print json['totalResults']
        
    def testFootPrintSearch(self):
        # footprint search
        p = Places()
        res = p.nameSearch(name=self.name,format='json')        
        json = simplejson.loads(res)
        place = json['features'][0]
        foot = p.footprintLookup(id=place['id'])
   
    def testNameAndFeatureSearch(self):
        p = Places()
        res = p.nameAndFeatureSearch(name=self.name,featureType='populated place')
         
    def testText(self):
        t = Text()
        text = open('test.txt').read()
        res = t.places(text) 

    def testTextJSON(self):
        t = Text()
        text = open('test.txt').read()
        res = t.places(text,{'format':'json'})
if __name__ == '__main__':
    unittest.main()

